// ANSI includes
#include <cstdio>
#include <cstdlib>
#include <cstring>

// Application includes
#include "Gauges.h"

// Constant definitions
const int max_temp = 35;

Gauges::Gauges(BRect frame, const char *name, uint32 resizeMask, uint32 flags)
		:	BView(frame, name, resizeMask, flags),
			init_(false),
			state_(0),
			lm78_()
{	Pulse();
	SetViewColor(189,186,189);
	SetLowColor(189,186,189);
	BRect rect(Bounds());
	rect.OffsetTo(B_ORIGIN);
	rect.top = rect.bottom - 7;
	rect.left = rect.right - 7;
	SetFontSize(10);
}

Gauges::Gauges(BMessage *message)
	: BView(message)
{
}


Gauges::~Gauges()
{
}

// archiving overrides
Gauges* Gauges::Instantiate(BMessage *data)
{	if (!validate_instantiation(data, "Gauges"))
		{	return NULL;
		};
	return new Gauges(data);
}

status_t Gauges::Archive(BMessage *data, bool deep) const
{	uint32 winresizingmode = ResizingMode();
	((BView*)this)->BView::SetResizingMode(B_FOLLOW_LEFT|B_FOLLOW_TOP);
	BView::Archive(data, deep);
	((BView*)this)->SetResizingMode(winresizingmode);

	data->AddString("add_on", "application/x-vnd.temp");
	return B_NO_ERROR;
}


void Gauges::Draw(BRect rect)
{	BView::Draw(rect);
	if (state_ > 2) state_ = 0;

	char	buffer[64]; memset(buffer,0,64);
	switch (state_)
		{	case 0:
				{	if (lm78_.MotherboardTemp == 0)
						{	sprintf(buffer,"? °C");
						}
					else
						{	sprintf(buffer,"%d° C",(int) lm78_.MotherboardTemp);
							if ((int) lm78_.MotherboardTemp >= max_temp)
								{	system("shutdown");
								};
						}
					break;
				}
			case 1:
				{	sprintf(buffer,"%d",(int) lm78_.CPUFan);
					break;
				}
			case 2:
				{	sprintf(buffer,"%1.1f V",lm78_.VCore);
					break;
				}
		}
			
	rgb_color c = { 189, 182, 189 };
	SetHighColor(c);
	FillRect(Bounds());
	rgb_color b = { 0, 0, 0 };
	SetHighColor(b);
	MovePenTo(2,13);
	DrawString(buffer);
}

void Gauges::MessageReceived(BMessage *message)
{	BView::MessageReceived(message);
}

void Gauges::AttachedToWindow()
{	Window()->SetPulseRate(200000);
}


void Gauges::Pulse()
{	if (!init_)
		{	init_ = (lm78_.InitCheck() == B_OK);
		}
	else
		{	lm78_.Update();
		};
	Draw(Bounds());
}

void Gauges::MouseDown(BPoint /*point*/)
{	state_++;
	if (state_ > 2) state_ = 0;
	Pulse();
}

